// -----------------------------------------------------------------------------
// Init
// -----------------------------------------------------------------------------

const http = require('http')
const debug = require('debug')('sandbox:client')
const serialize = require('serialize-javascript')

// -----------------------------------------------------------------------------
// Main
// -----------------------------------------------------------------------------

const sendScriptJob = function (scriptObj, callback) {
  var timeA = Date.now()
  var heapStart = process.memoryUsage().rss

  // to send over http, must serialize object with functions
  // TODO: serialize-javascript danger: uses eval() to parse
  var postData = serialize(scriptObj)

  var options = {
    hostname: 'localhost',
    port: 42121,
    method: 'POST',
    headers: {
      'Content-Type': 'application/json'
    }
  }

  var request = http.request(options, function (response) {
    debug('response statusCode ', response.statusCode)

    var body = ''

    response.on('data', function (chunk) {
      body += chunk
    })

    response.on('end', function () {
      body = JSON.parse(body)
      debug('response body', body)
      if (response.statusCode === 200) {
        callback(null, body)
      } else {
        callback(body, null)
      }

      debug('######### SANDBOX RESPONSE TIME ######### ', Date.now() - timeA)
      debug('client process memoryUsage increase ' + ((process.memoryUsage().rss - heapStart) / 1000000))
    })
  })

  // timeout on loops requires error event after data event
  request.on('error', function (e) {
    // socket hangup reported here - socket hang up
    debug('request - Got error:', e)
    callback({ message: e.message, statusCode: 500 }, null)
  })

  // write data to request body
  request.write(postData)
  request.end()
}

// -----------------------------------------------------------------------------
// EXPORTS
// -----------------------------------------------------------------------------

module.exports = sendScriptJob
