// -----------------------------------------------------------------------------
// Init
// -----------------------------------------------------------------------------

const cluster = require('cluster')
const debug = require('debug')('sandbox:cluster')
const _ = require('lodash')

// -----------------------------------------------------------------------------
// Main
// -----------------------------------------------------------------------------
const timeoutMSecs = 5 * 1000

var listOfWorkers = []
var timeoutIDs = []

var clearTimeouts = function () {
  _.each(timeoutIDs, function (id) {
    clearTimeout(id)
  })
}

const startCluster = function (numCPUs, startClusterCallback) {
  var pjson = require('./package.json')
  console.log('SANDBOX CLUSTER STARTED v' + pjson.version)

  if (!numCPUs) {
    if (process.env.DEBUG || process.env.TESTING) {
      numCPUs = 1
    } else {
      numCPUs = require('os').cpus().length
    }
  }

  const setUpOneFork = function () {
    var worker = cluster.fork()

    var timeoutID // outside worker scope

    worker.on('message', function (msg) {
      debug('worker message: ', msg)

      // #set external kill switch
      // msg contains the processId
      // means the worker has started a sandbox
      // if time runs out, kill the process (cannot kill reliably within process)

      // on task assigned to worker, worker returns with process.pid
      if (msg.event === 'started') { // is any kind of number
       // without the try catch there may be a channel closed error
        timeoutID = setTimeout(function () {
          debug(`vm timeout`, msg.pid)
          // check if exists for a processID
          // http://stackoverflow.com/questions/14390930/how-to-check-if-an-arbitrary-pid-is-running-using-node-js
          // if( process.kill( msg.pid , 0) ){
          worker.send({ pid: msg.pid, event: 'notifyTimeout' })

            // debug('killing process ', msg.pid)
            // process.kill(msg.pid )

            // use kill switch if the process is unresponsive
          var subtimeout = setTimeout(function () {
            // tests produce Error: kill ESRCH
            // if the process is not available and kill is attmpted you get Error: kill ESRCH
            debug('kill worker ', msg.pid)
            process.kill(msg.pid, 'SIGKILL')
          }, 100) // give it one more second to receive all messages
          timeoutIDs.push(subtimeout)

          // }
        }, timeoutMSecs)
        timeoutIDs.push(timeoutID)
      }

      if (msg.event === 'complete') {
        clearTimeouts()
      }

      //  when at least one worker http server is listening.
      if (msg.event === 'ready' && startClusterCallback) {
        // return that at least one server is running.
        startClusterCallback()
        // run once, empty the function
        startClusterCallback = function () {}
      }
    })

    worker.on('SIGKILL', function (code, signal) {
      debug('worker ' + worker.process.pid + ' died.')

      // start a new worker
      var newWorker = setUpOneFork() // cluster.fork();
      debug('worker ' + newWorker.process.pid + ' born.')

      clearTimeouts()
    })

    return worker.process.pid
  }

  cluster.setupMaster({
    exec: __dirname + '/http-worker.js'
  })

  if (cluster.isMaster) {
    // CREATE A CLUSTER OF FORKED SCRIPTS, ONE PER CPU
    debug(`Master ${process.pid} is running`)

    for (var i = 0; i < numCPUs; i++) {
      var workerPid = setUpOneFork()
      listOfWorkers.push(workerPid)
    }
  }

  return listOfWorkers // process.pid
}

const killCluster = function (callback) {
  debug('disconnect cluster - kill all workers')
  // debug('listOfWorkers', listOfWorkers);
  cluster.disconnect(callback)

  clearTimeouts()

  // _.each( listOfWorkers, function(pid){
  //   kill( pid )
  //   setTimeout(function(){
  //     debug( 'process alive: ',  process.kill( pid, 0 ) );
  //
  //   }, 100);
  //
  // })

  // debug('pidArray', pidArray);
}

// -----------------------------------------------------------------------------
// EXPORTS
// -----------------------------------------------------------------------------

module.exports = { startCluster, killCluster }
