// -----------------------------------------------------------------------------
// Init
// -----------------------------------------------------------------------------

const http = require('http')
const sandbox = require('./sandbox.js')
const debug = require('debug')('sandbox:worker')

// -----------------------------------------------------------------------------
// Main
// -----------------------------------------------------------------------------

var server = http.createServer(function (req, response) {
  // send the process id for timeout function
  debug(`Worker ${process.pid} is running`)

  // notify the master node what the processid for the killswitch
  process.send({ pid: process.pid, event: 'started' })

  var body = ''
  req.on('data', function (data) {
    // debug('http-worker.js - request buffer received ', data)
    body += data
  })

  req.on('end', function () {
    // body created with 'serialize-javascript' lib
    var scriptObj = eval('(' + body + ')')

    // if a killswitch notifyTimeout message is received while running
    // TODO: process.on()  creates a leak when used inside another event emitter.
    var notifyTimoutHandler = function (m) {
      if (m.pid === process.pid && m.event === 'notifyTimeout') {
        response.writeHead(500, { 'Content-Type': 'application/json'})
        // TODO: No console logs available to user on killswitch timeout
        response.write(JSON.stringify({ status: 'TIMEOUT', result: null, console: [] })) // errors were not being reported
        response.end()
      }
    }

    process.on('message', notifyTimoutHandler)

    // run sanbox in request
    sandbox(scriptObj.script, scriptObj.libs, scriptObj.context, function (err, res) {
      // prevent worker coordinator to not run kill switch
      process.send({ pid: process.pid, event: 'complete' })
      // if sandbox was successful, then remove the killswitch listener
      process.removeListener('message', notifyTimoutHandler)

      if (err) {
        debug('sandbox err:', err)
        response.writeHead(500, { 'Content-Type': 'application/json'})
      } else {
        debug('sandbox res:', res)
        response.writeHead(200, { 'Content-Type': 'application/json'})
      }

      var output = res || err
      response.write(JSON.stringify(output)) // errors were not being reported
      response.end()
    })
  })
})

server.listen(42121)

server.on('listening', function () {
  // debug('http-worker.js - server listening ')
  process.send({ pid: process.pid, event: 'ready'})
})

server.on('error', (err, socket) => {
  debug('http-worker.js -  error - unknown ' + err)
})

process.on('uncaughtException', function (err) {
  // process.send("http-worker.js - uncaughtException " + err.stack);
  debug('uncaughtException event', err.stack)
  // process.exit();
})
